#! /bin/sh
### ====================================================================
###  @UNIX-shell-file{
###     author          = "Nelson H. F. Beebe",
###     version         = "3.0.0",
###     date            = "18 May 2001",
###     time            = "05:05:04 MDT",
###     filename        = "foo.sh",
###     copyright       = "Copyright (c) 1990, 1993, 2001 Nelson H. F.
###                        Beebe. This code is licensed under the GNU
###                        General Public License, version 2 or
###                        later.",
###     address         = "Center for Scientific Computing
###                        University of Utah
###                        Department of Mathematics, 322 INSCC
###                        155 S 1400 E RM 233
###                        Salt Lake City, UT 84112-0090
###                        USA",
###     telephone       = "+1 801 581 5254",
###     FAX             = "+1 801 585 1640, +1 801 581 4148",
###     URL             = "http://www.math.utah.edu/~beebe",
###     checksum        = "13004 268 940 10529",
###     email           = "beebe@math.utah.edu, beebe@acm.org,
###                        beebe@computer.org, beebe@ieee.org
###                        (Internet)",
###     codetable       = "ISO/ASCII",
###     keywords        = "Fortran; preprocessor; SFTRAN3",
###     supported       = "yes",
###     docstring       = "This is a shell script wrapper to run fpp
###                        with implementation-dependent predefined
###                        symbols.  See fpp(1) for details.
###
###                        All symbol definitions are determined
###                        DYNAMICALLY, so that this script can be
###                        identical on all UNIX-like platforms, and so
###                        that it does not require updating when the
###                        operating system is upgraded.
###
###                        For each system, there should be uppercase
###                        symbols with one leading underscore defined
###                        for at least these:
###
###                             * the operating system family (normally,
###                               _OS_UNIX);
###
###                             * the vendor operating system system
###                               (_AIX, _HPUX, _SOLARIS, _SUNOS, ...);
###
###                             * the operating system level or
###                               revision number (3.2.1, 4.1, ...) as
###                               the value of _OS_LEVEL;
###
###                             * the host architecture (_ALPHA,
###                               _IBM_3090, _SPARC, ..., etc.);
###
###                             * the endianness (_BIG_ENDIAN or
###                               _LITTLE_ENDIAN)
###
###                             * _IEEE_754 if the arithmetic is
###                               IEEE-754 style (though sometimes not
###                               fully conformant);
###
###                             * _NAMELIST if Fortran NAMELIST
###                               statements are supported.
###
###                        Revisions:
###
###                        [14-May-2001] -- update for GNU
###                                         autoconfiguration
###                        [18-Mar-1991] -- original version
###
###                        The checksum field above contains a CRC-16
###                        checksum as the first value, followed by the
###                        equivalent of the standard UNIX wc (word
###                        count) utility output of lines, words, and
###                        characters.  This is produced by Robert
###                        Solovay's checksum utility.",
###  }
### ====================================================================

warning=no

### --------------------------------------------------------------------
### NB: These case values have been adapted from an old version of this
###     software and several of these environments no longer exist, so
###     it has not been possible to check the uname output for them.
###     This should be harmless, since presumably they are all retired
###     anyway.
### --------------------------------------------------------------------

osname="`uname -s || echo unknown`"
osname="`echo $osname | tr -c 'a-zA-Z0-9.\n-' '_' `"
osrev="`uname -r || echo unknown`"              # all but IBM AIX have this

case "$osname" in
	AIX370*)        DEFINES="-D_OS_UNIX -D_AIX370 -D_ARCH=IBM-3090"
			DEFINES="${DEFINES} -D_IBM_3090 -D_BIG_ENDIAN"
			DEFINES="${DEFINES} -D_NAMELIST"
			;;
	AIX*)           DEFINES="-D_OS_UNIX -D_AIX -D_ARCH=PowerPC"
			DEFINES="${DEFINES} -D_PPC -D_IBM_RS_6000"
			DEFINES="${DEFINES} -D_BIG_ENDIAN -D_IEEE_754"
			DEFINES="${DEFINES} -D_NAMELIST"
			osrev=`oslevel || true`
			;;
	Convex*)        DEFINES="-D_OS_UNIX -D_BSD -D_ARCH=Convex"
			DEFINES="${DEFINES} -D_CONVEX -D_BIG_ENDIAN"
			DEFINES="${DEFINES} -D_NAMELIST"
			;;
	Cray*)          DEFINES="-D_OS_UNIX -D_ARCH=Cray -D_CRAY"
			DEFINES="${DEFINES} -D_BIG_ENDIAN"
			;;
	Darwin*)        DEFINES="-D_OS_UNIX -D_DARWIN -D_MACOSX"
			DEFINES="${DEFINES} -D_ARCH=PowerPC -D_PPC"
			DEFINES="${DEFINES} -D_BIG_ENDIAN -D_IEEE_754"
			DEFINES="${DEFINES} -D_NAMELIST"
			;;
	FreeBSD*)       DEFINES="-D_OS_UNIX -D_FREEBSD -D_ARCH=x86"
			DEFINES="${DEFINES} -D_LITTLE_ENDIAN"
			DEFINES="${DEFINES} -D_IEEE_754 -D_NAMELIST"
			;;
	Gould*)         DEFINES="-D_OS_UNIX -D_ARCH=Gould -D_GOULD"
			DEFINES="${DEFINES} -D_BIG_ENDIAN"
			;;
	Linux*)         DEFINES="-D_OS_UNIX -D_LINUX -D_GNU_LINUX"
			DEFINES="${DEFINES} -D_IEEE_754"
			cpu=`uname -m || echo unknown`
			case "$cpu" in
				9000/*)	# Hewlett-Packard 9000/7xx and 9000/8xx
					DEFINES="${DEFINES} -D_BIG_ENDIAN"
					DEFINES="${DEFINES} -D_ARCH=PA-RISC"
					DEFINES="${DEFINES} -D_PA_RISC"
					;;
				alpha*)
					DEFINES="${DEFINES} -D_LITTLE_ENDIAN"
					DEFINES="${DEFINES} -D_ARCH=Alpha"
					DEFINES="${DEFINES} -D_ALPHA"
					;;
				i386* | i486* | i586* | i686* )
					DEFINES="${DEFINES} -D_LITTLE_ENDIAN"
					DEFINES="${DEFINES} -D_ARCH=$cpu"
					DEFINES="${DEFINES} -D_X86"
					DEFINES="${DEFINES} -D_`echo $cpu | tr a-z A-Z`"
					;;
				ia64*)
					DEFINES="${DEFINES} -D_LITTLE_ENDIAN"
					DEFINES="${DEFINES} -D_ARCH=IA-64"
					DEFINES="${DEFINES} -D_IA64"
					;;
				mips*)
					DEFINES="${DEFINES} -D_BIG_ENDIAN"
					DEFINES="${DEFINES} -D_ARCH=MIPS"
					DEFINES="${DEFINES} -D_MIPS"
					;;
				ppc*)
					DEFINES="${DEFINES} -D_BIG_ENDIAN"
					DEFINES="${DEFINES} -D_ARCH=PowerPC"
					DEFINES="${DEFINES} -D_PPC"
					;;
				sparc*)
					DEFINES="${DEFINES} -D_BIG_ENDIAN"
					DEFINES="${DEFINES} -D_ARCH=SPARC"
					DEFINES="${DEFINES} -D_SPARC"
					;;
				*)
					warning=yes ;;
			esac
			;;
	HPPA*)          DEFINES="-D_OS_UNIX -D_HPPA -D_BIG_ENDIAN"
			DEFINES="${DEFINES} -D_ARCH=PA-RISC -D_HPPA"
			DEFINES="${DEFINES} -D_IEEE_754 -D_NAMELIST"
			;;
	HP-UX*)         DEFINES="-D_OS_UNIX -D_HPUX -D_BIG_ENDIAN"
			DEFINES="${DEFINES} -D_ARCH=PA-RISC -D_HPPA"
			DEFINES="${DEFINES} -D_IEEE_754 -D_NAMELIST"
			;;
	IRIX64*)        DEFINES="-D_OS_UNIX -D_IRIX -D_IRIX64"
			DEFINES="${DEFINES} -D_BIG_ENDIAN -D_ARCH=MIPS"
			DEFINES="${DEFINES} -D_MIPS -D_IEEE_754 -D_NAMELIST"
			;;
	IRIX)           DEFINES="-D_OS_UNIX -D_IRIX -D_BIG_ENDIAN"
			DEFINES="${DEFINES} -D_ARCH=MIPS -D_MIPS"
			DEFINES="${DEFINES} -D_IEEE_754 -D_NAMELIST"
			;;
	MIPS*)          DEFINES="-D_OS_UNIX -D_MIPS -D_BIG_ENDIAN"
			DEFINES="${DEFINES} -D_ARCH=MIPS -D_MIPS"
			DEFINES="${DEFINES} -D_IEEE_754 -D_NAMELIST"
			;;
	NetBSD*)        DEFINES="-D_OS_UNIX -D_NETBSD -D_ARCH=x86"
			DEFINES="${DEFINES} -D_LITTLE_ENDIAN"
			DEFINES="${DEFINES} -D_IEEE_754 -D_NAMELIST"
			;;
	NeXT*)          DEFINES="-D_OS_UNIX -D_NEXT -D_BIG_ENDIAN"
			DEFINES="${DEFINES} -D_ARCH=Motorola-68K"
			DEFINES="${DEFINES} -D_M68K -D_IEEE_754 -D_NAMELIST"
			;;
	OpenBSD*)       DEFINES="-D_OS_UNIX -D_OPENBSD -D_ARCH=x86"
			DEFINES="${DEFINES} -D_LITTLE_ENDIAN"
			DEFINES="${DEFINES} -D_IEEE_754 -D_NAMELIST"
			;;
	OSF1*)          DEFINES="-D_OS_UNIX -D_OSF1 -D_BIG_ENDIAN"
			DEFINES="${DEFINES} -D_ARCH=Alpha -D_ALPHA"
			DEFINES="${DEFINES} -D_IEEE_754 -D_NAMELIST"
			;;
	Rhapsody*)      DEFINES="-D_OS_UNIX -D_RHAPSODY -D_MACOSX"
			DEFINES="${DEFINES} -D_ARCH=PowerPC -D_PPC"
			DEFINES="${DEFINES} -D_BIG_ENDIAN -D_IEEE_754"
			DEFINES="${DEFINES} -D_NAMELIST"
			;;
	Stardent*)      DEFINES="-D_OS_UNIX -D_BIG_ENDIAN"
			DEFINES="${DEFINES} -D_ARCH=Stardent"
			DEFINES="${DEFINES} -D_STARDENT"
			DEFINES="${DEFINES} -D_STARDENT_1500"
			DEFINES="${DEFINES} -D_IEEE_754 -D_NAMELIST"
			;;
	Sun386*)        DEFINES="-D_OS_UNIX -D_SUNOS -D_ARCH=x86 -D_SUN"
			DEFINES="${DEFINES} -D_`/bin/arch | tr a-z A-Z`"
			DEFINES="${DEFINES} -D_LITTLE_ENDIAN"
			DEFINES="${DEFINES} -D_IEEE_754 -D_NAMELIST"
			;;
	Sun[34]*)       DEFINES="-D_OS_UNIX -D_SUNOS -D_ARCH=SPARC"
			DEFINES="${DEFINES} -D_SUN -D_`/bin/arch | tr a-z A-Z`"
			DEFINES="${DEFINES} -D_BIG_ENDIAN -D_IEEE_754"
			DEFINES="${DEFINES} -D_NAMELIST"
			;;
	SunOS*)         DEFINES="-D_OS_UNIX -D_SUNOS -D_ARCH=SPARC"
			DEFINES="${DEFINES} -D_SUN -D_`/bin/arch | tr a-z A-Z`"
			DEFINES="${DEFINES} -D_BIG_ENDIAN -D_IEEE_754"
			DEFINES="${DEFINES} -D_NAMELIST"
			;;
	Ultrix*)        case "`uname -m || true`" in
				VAX*)   DEFINES="-D_OS_UNIX -D_BSD"
					DEFINES="${DEFINES} -D_ARCH=VAX"
					DEFINES="${DEFINES} -D_ULTRIX"
					DEFINES="${DEFINES} -D_LITTLE_ENDIAN"
					DEFINES="${DEFINES} -D_VAX -D_NAMELIST"
					;;
				*)      DEFINES="-D_OS_UNIX -D_BSD"
					DEFINES="${DEFINES} -D_ARCH=MIPS"
					DEFINES="${DEFINES} -D_ULTRIX"
					DEFINES="${DEFINES} -D_LITTLE_ENDIAN"
					DEFINES="${DEFINES} -D_IEEE_754 -D_NAMELIST"
					;;
			esac
			;;
	*)              DEFINES="-D_OS_UNIX -D_ARCH=unknown -D_NAMELIST"
			warning=yes
			;;
esac

DEFINES="-D_OS_NAME=$osname -D_OS_LEVEL=$osrev ${DEFINES}"

test warning = yes && echo WARNING: unrecognized host system: defaulting to ${DEFINES}

if test -n "$DEBUGFPP"
then
	echo "DEBUG: Date:        " `date`
	echo "DEBUG: Directory:   " $PWD
	echo "DEBUG: Hostname:    " `hostname`
	echo "DEBUG: Machine type:" `machinetype || true`
	echo "DEBUG: System:      " `uname -a`

	echo "DEBUG: $0: DEFINES ="
	echo ${DEFINES} | tr ' ' '\n' | LANG=C sort | awk '{print "DEBUG:\t\t\t\t\t" $0}'
fi

### We require fpp.exe to be in the same directory as this script:
`dirname $0`/fpp.exe ${DEFINES} "$@"
